function GICOV_DP_snake
% Dynamic Programming Snake with GICOV Criterion
% N. Ray, S. Acton, H. Zhang, Seeing through clutter: Snake computation
% with dynamic programming for particle segmentation, ICPR 2012.
% Program writen by Nilanjan Ray
% Last modified on July 2012

close all;
clear all;


%%%%%%%%%%%%%%%%%%%%%%%%% Parameter Setting %%%%%%%%%%%%%%%%%%%%%

% How many radial lines?
N =250;

% search radius range
search_rad = (15:1:100)';

% snake smoothness parameter
delta=1;

% How many tsteps for GICOV-DP
nStep=20;

% Object brighter than surrounding? if yes set 1, else set 0
IsBrighter=1;

% if you need user interactive segmentation, set the follwoing to 1
user_interactive=0;

% Prefiltering the image: Gaussian smoothing parameter
sig=1.5;

%%%%%%%%%%%%%%%%%%%%%%%%% End Parameter Setting %%%%%%%%%%%%%%%%%%%%%


% read example image
I = double(imread('crop25.bmp'));

figure(1),imagesc(I),colormap(gray),axis image;

if user_interactive,
    [cx,cy] = ginput(1);
    cx=round(cx);cy=round(cy);
%     disp([cx cy]);
else
    cx = 284;    cy=95;
end
hold on, plot(cx,cy,'*'),hold off;

% construct radial lines
theta = linspace(0,2*pi,N+1);
theta = theta(1:N);

[R,T] = meshgrid(search_rad,theta);
ix = R.*cos(T) + cx;
iy = R.*sin(T) + cy;


I(:) = imfilter(I,fspecial('Gaussian',[2*round(3*sig)+1 2*round(3*sig)+1],sig),'same','conv','replicate');
[sIx,sIy] = gradient(I);
iIx = interp2(sIx,ix,iy,'*linear',0);
iIy = interp2(sIy,ix,iy,'*linear',0);

% directional gradient on radial lines
if IsBrighter,
    g = -sqrt(iIx.*iIx+iIy.*iIy).*sign(iIx.*cos(T)+iIy.*sin(T)); 
else
    g = sqrt(iIx.*iIx+iIy.*iIy).*sign(iIx.*cos(T)+iIy.*sin(T)); % directional gradient
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% DP snake with max directional grad%%%%

% call DP snake with maximum directional gradient criterion
ind=gicov_dyn_snake(max(g(:)),g,delta);
X = ix(ind);
Y = iy(ind);

figure(1),hold on; plot([X; X(1)],[Y; Y(1)],'r');hold off; drawnow;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% DP snake with max GICOV %%%%%%%%%%%%%%

mg = -1e10; mind = 0;
maxg=max(g(:));
gm = linspace(0,maxg,nStep);
old_gicov=-1e10;cnt=0;
for i=1:nStep,
        ind=gicov_dyn_snake(gm(i),g,delta);
        this_gicov = mean(g(ind))/std(g(ind));
        if this_gicov>mg,
            mg = this_gicov;
            clear mind;
            mind = ind;
        end
        clear ind;
    disp([gm(i) this_gicov]);
    % a stopping criteria - heuristic
    if old_gicov>this_gicov,
        cnt = cnt + 1;
    else
        cnt=0;
    end
    if cnt==3,
        break;
    end
    old_gicov=this_gicov;
end

Xg = ix(mind);
Yg = iy(mind);

figure(1),hold on; plot(Xg,Yg,'g');
title('Red contour: Max Grad snake, Green contour: GICOV-DP snake.');

function ind=gicov_dyn_snake(gm,g,delta)
% dynamic programing for computing gicov (closed contour) snake
% most stable version - works in O(NM^2) time

[N,M] = size(g);

OldValues =  1e10*ones(M,M);
Indices = zeros(N-2,M,M);

% forward pass to build the value functions and indices
for i=1:M,
    for j=max(1,i-delta):min(M,i+delta),
        OldValues(i,j)=0;
    end
end
for n=1:N-2,
    Values = 1e10*ones(M,M);
    for i=1:M,
        for k=1:M,
            minv=1e10;minind=0;
            for j=max(1,k-delta):min(M,k+delta),
                thisv = OldValues(i,j) + abs(gm-g(n+1,j));
                if minv>thisv,
                    minv=thisv;
                    minind=j;
                end
            end
            Values(i,k) = minv;
            Indices(n,i,k) = minind;
        end
    end
    clear OldValues;
    OldValues = Values;
    clear Values;
end

% backtrack
ind = zeros(N,1);
minv=1e10;
for i=1:M,
    for j=max(1,i-delta):min(M,i+delta),
        thisv = OldValues(i,j) + abs(gm-g(1,i)) + abs(gm-g(N,j));
        if minv>thisv,
            minv=thisv;
            ind(1) = i;
            ind(N) = j;
        end
    end
end
for n=N-1:-1:2,
    ind(n) = Indices(n-1,ind(1),ind(n+1));
end
ind(:) = N*(ind-1) + (1:N)';
